/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "assert/assert.h"
#include "util/memory.h"
#include "entity.h"

entity_t ENTITIES[ENTITY_COUNT];

entitycallbacks_t ENTITY_CALLBACKS[ENTITY_TYPE_COUNT] = {
    [ENTITY_TYPE_NULL] = {
      .init = NULL
    },
    
    [ENTITY_TYPE_PLAYER] = {
      .init = playerInit
    },
};



void entityInit(entity_t *entity, const entitytype_t type) {
  assertNotNull(entity, "Entity is NULL");
  assertTrue(type < ENTITY_TYPE_COUNT, "Invalid entity type");
  assertNotNull(ENTITY_CALLBACKS[type].init, "Entity type has no init");

  memoryZero(entity, sizeof(entity_t));
  entity->type = type;
  ENTITY_CALLBACKS[type].init(entity);
}